Package: BREW3R.r
Type: Package
Title: R package associated to BREW3R
Version: 1.9.0
Authors@R: 
    person(given = "Lucille",
           family = "Lopez-Delisle",
           role = c("aut", "cre"),
           email = "lucille.delisle@epfl.ch",
           comment = c(ORCID = "0000-0002-1964-4960"))
Description: This R package provide functions that are used in the BREW3R
    workflow. This mainly contains a function that extend a gtf as GRanges using
    information from another gtf (also as GRanges). The process allows to extend
    gene annotation without increasing the overlap between gene ids.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.1
Imports: GenomicRanges, methods, rlang, S4Vectors, utils
Suggests: testthat (>= 3.0.0), IRanges, knitr, rmarkdown, BiocStyle,
        rtracklayer
biocViews: GenomeAnnotation
Config/testthat/edition: 3
URL: https://github.com/lldelisle/BREW3R.r
BugReports: https://github.com/lldelisle/BREW3R.r/issues/
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/BREW3R.r
git_branch: devel
git_last_commit: ccdaccc
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:35:37 UTC; biocbuild
Author: Lucille Lopez-Delisle [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1964-4960>)
Maintainer: Lucille Lopez-Delisle <lucille.delisle@epfl.ch>
